@@ -1,17 +1,18 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
+//          Aevol - An in silico experimental evolution platform
 //
-// Copyright (C) 2004 LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
+// ****************************************************************************
 // 
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,14 +20,10 @@
 // GNU General Public License for more details.
 // 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
-
-/** \class
- *  \brief
- */
  
 const char* DEFAULT_PARAM_FILE_NAME = "param.in";
  
@@ -226,6 +223,16 @@ int main( int argc, char* argv[] )
       delete env_axis_segment_boundaries;
       delete env_axis_features;
     }
+    else if ( strcmp( line->words[0], "POPULATION_SIZE") == 0 )
+    {
+    	pop->set_nb_indivs(atol( line->words[1] ) );
+      printf("\tChange of population size to %d\n",atol( line->words[1] ));
+    }
+    else if ( strcmp( line->words[0], "SELECTION_PRESSURE") == 0 )
+    {
+    	sel->set_selection_pressure(atof( line->words[1] ) );
+      printf("\tChange of selection pressure to %f\n",atof( line->words[1] ));
+    }
     else if ( strcmp( line->words[0], "POINT_MUTATION_RATE" ) == 0 )
     {
       pop->set_overall_point_mutation_rate( atof( line->words[1] ) );
@@ -266,6 +273,16 @@ int main( int argc, char* argv[] )
       pop->set_overall_inversion_rate( atof( line->words[1] ) );
       printf("\tChange of overall inversion to %f\n",atof( line->words[1] ));
     }
+    else if ( strcmp( line->words[0], "TRANSFER_INS_RATE" ) == 0 )
+    {
+      pop->set_overall_transfer_ins_rate( atof( line->words[1] ) );
+      printf("\tChange of overall transfer insertion rate to %f\n",atof( line->words[1] ));
+    }
+    else if ( strcmp( line->words[0], "TRANSFER_REPL_RATE" ) == 0 )
+    {
+      pop->set_overall_transfer_repl_rate( atof( line->words[1] ) );
+      printf("\tChange of overall transfer replacement rate to %f\n",atof( line->words[1] ));
+    }
     else if ( strcmp( line->words[0], "ENV_ADD_GAUSSIAN" ) == 0 )
     {
       if ( env_change)
@@ -429,7 +446,6 @@ int main( int argc, char* argv[] )
   {
     env->build();
   }
-  //printf("%e\n", pop->get_best()->get_point_mutation_rate());
   
   // 9) Save the modified experiment
   printf("Save the modified experiment into backup...\t");
@@ -571,7 +587,7 @@ void change_based_on_non_coding_bases_of_best_individual_ancestor(ae_population*
     #endif
     char pop_file_name[255];
     sprintf( pop_file_name, POP_FNAME_FORMAT, num_gener  );
-    tree = new ae_tree( exp_m, pop_file_name, tree_file_name );
+    tree = new ae_tree( exp_m, tree_file_name );
     ae_replication_report * reports = new ae_replication_report( *(tree->get_report_by_rank(num_gener, tree->get_nb_indivs( num_gener )) ));
     ancestor_index = reports->get_id();
     for (int32_t i = 1; i <= backup_step ; i++ )
@@ -592,7 +608,7 @@ void change_based_on_non_coding_bases_of_best_individual_ancestor(ae_population*
         }
         
         delete tree;
-        tree = new ae_tree( exp_m, pop_file_name, tree_file_name );
+        tree = new ae_tree( exp_m, tree_file_name );
       }
 
       reports = new ae_replication_report( *(tree->get_report_by_index(num_gener-i, ancestor_index)) );
