@@ -1,27 +1,28 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
-//
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+//          Aevol - An in silico experimental evolution platform
 //
+// ****************************************************************************
+// 
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-//*****************************************************************************
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
+// ****************************************************************************
 
 
 /*!
@@ -111,8 +112,10 @@ class ae_exp_manager : public ae_object
   
     // -------------------------------------------------------- Global settings
     inline bool   get_with_HT( void ) const;
+    inline bool   get_repl_HT_with_close_points ( void ) const;
     inline double get_HT_ins_rate( void ) const;
     inline double get_HT_repl_rate( void ) const;
+    inline double get_repl_HT_detach_rate( void ) const;
     
     // The ability to own a plasmid is a property of the individuals (_allow_plasmids) because it is used during mutations
     // However there is also a property of the experimental setup (_with_plasmids) that indicates whether plasmids are used because we need this during replication and during loading/writting
@@ -358,6 +361,11 @@ inline bool ae_exp_manager::get_with_HT( void ) const
   return get_exp_s()->get_with_HT();
 }
 
+inline bool ae_exp_manager::get_repl_HT_with_close_points ( void ) const
+{
+  return get_exp_s()->get_repl_HT_with_close_points();
+}
+
 inline double ae_exp_manager::get_HT_ins_rate( void ) const
 {
   return get_exp_s()->get_HT_ins_rate();
@@ -368,6 +376,11 @@ inline double ae_exp_manager::get_HT_repl_rate( void ) const
   return get_exp_s()->get_HT_repl_rate();
 }
 
+inline double ae_exp_manager::get_repl_HT_detach_rate( void ) const
+{
+  return get_exp_s()->get_repl_HT_detach_rate();
+}
+
 inline bool ae_exp_manager::get_with_plasmids( void ) const
 {
   return get_exp_s()->get_with_plasmids();
