@@ -1,32 +1,28 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
-//
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+//          Aevol - An in silico experimental evolution platform
 //
+// ****************************************************************************
+// 
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-//*****************************************************************************
-
-
-/** \class
- *  \brief
- */
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
+// ****************************************************************************
 
 
 #ifndef __AE_INDIVIDUAL_H__
@@ -165,6 +161,31 @@ class ae_individual : public ae_object
     inline double get_translocation_proportion( void ) const;
     inline double get_inversion_proportion( void ) const;
     
+    // Transfer
+    inline bool   get_with_4pts_trans( void ) const;
+    inline bool   get_with_HT( void ) const;
+    inline bool   get_repl_HT_with_close_points( void ) const;
+    inline double get_HT_ins_rate( void ) const;
+    inline double get_HT_repl_rate( void ) const;
+    inline double get_repl_HT_detach_rate( void ) const;
+    
+    // ------------------------------------------------------------ Alignements
+    inline bool  get_with_alignments(void) const;
+    inline ae_align_fun_shape get_align_fun_shape( void ) const;
+    inline double  get_align_sigm_lambda( void ) const;
+    inline int16_t get_align_sigm_mean( void ) const;
+    inline int16_t get_align_lin_min( void ) const;
+    inline int16_t get_align_lin_max( void ) const;
+    
+    // Maximum shift of one seq on the other
+    inline int16_t get_align_max_shift( void ) const;
+    // Work zone half length
+    inline int16_t get_align_w_zone_h_len( void ) const;
+    // Corresponding residues match bonus
+    inline int16_t get_align_match_bonus( void ) const;
+    // Corresponding residues mismatch cost
+    inline int16_t get_align_mismatch_cost( void ) const;
+    
     
     // ----------------------------------------------- Phenotypic stochasticity
     inline bool get_with_stochasticity( void ) const;
@@ -239,6 +260,15 @@ class ae_individual : public ae_object
     inline void set_translocation_proportion( double translocation_proportion);
     inline void set_inversion_proportion( double inversion_proportion);
     
+    // Transfer
+    inline void set_with_4pts_trans( bool with_4pts_trans );
+    inline void set_with_alignments( bool with_alignments );
+    inline void set_with_HT( bool with_HT );
+    inline void set_repl_HT_with_close_points( bool repl_HT_with_close_points);
+    inline void set_HT_ins_rate( double HT_ins_rate );
+    inline void set_HT_repl_rate( double HT_repl_rate );
+    inline void set_repl_HT_detach_rate( double repl_HT_detach_rate );
+    
     // ----------------------------------------------- Phenotypic stochasticity
     inline void set_with_stochasticity( bool with_stoch );
 
@@ -1065,6 +1095,89 @@ inline double ae_individual::get_inversion_proportion( void ) const
   return _mut_params->get_inversion_proportion();
 }
 
+// ---------------------------------------------------------------- Transfer
+inline bool ae_individual::get_with_4pts_trans( void ) const
+{
+  return _mut_params->get_with_4pts_trans();
+}  
+
+inline bool ae_individual::get_with_HT( void ) const
+{
+  return _mut_params->get_with_HT();
+}  
+
+inline bool ae_individual::get_repl_HT_with_close_points( void ) const
+{
+  return _mut_params->get_repl_HT_with_close_points();
+}  
+
+inline double ae_individual::get_HT_ins_rate( void ) const
+{
+  return _mut_params->get_HT_ins_rate();
+}  
+
+inline double ae_individual::get_HT_repl_rate( void ) const
+{
+  return _mut_params->get_HT_repl_rate();
+}  
+
+inline double ae_individual::get_repl_HT_detach_rate( void ) const
+{
+  return _mut_params->get_repl_HT_detach_rate();
+}  
+
+    
+// ---------------------------------------------------------------- Alignements
+inline bool ae_individual::get_with_alignments(void) const
+{
+ return _mut_params->get_with_alignments();
+}
+
+inline ae_align_fun_shape ae_individual::get_align_fun_shape( void ) const
+{
+ return _mut_params->get_align_fun_shape();
+}
+
+inline double ae_individual::get_align_sigm_lambda( void ) const
+{
+ return _mut_params->get_align_sigm_lambda();
+}
+
+inline int16_t ae_individual::get_align_sigm_mean( void ) const
+{
+ return _mut_params->get_align_sigm_mean();
+}
+
+inline int16_t ae_individual::get_align_lin_min( void ) const
+{
+ return _mut_params->get_align_lin_min();
+}
+
+inline int16_t ae_individual::get_align_lin_max( void ) const
+{
+ return _mut_params->get_align_lin_max();
+}
+
+inline int16_t ae_individual::get_align_max_shift( void ) const
+{
+ return _mut_params->get_align_max_shift();
+}
+
+inline int16_t ae_individual::get_align_w_zone_h_len( void ) const
+{
+ return _mut_params->get_align_w_zone_h_len();
+}
+
+inline int16_t ae_individual::get_align_match_bonus( void ) const
+{
+ return _mut_params->get_align_match_bonus();
+}
+
+inline int16_t ae_individual::get_align_mismatch_cost( void ) const
+{
+ return _mut_params->get_align_mismatch_cost();
+}
+
 /*!
   TODO
 */
@@ -1277,6 +1390,42 @@ inline void ae_individual::set_inversion_proportion( double inversion_proportion
   _mut_params->set_inversion_proportion(inversion_proportion);
 }
 
+inline void ae_individual::set_with_4pts_trans( bool with_4pts_trans )
+{
+	_mut_params->set_with_4pts_trans(with_4pts_trans);
+}
+
+inline void ae_individual::set_with_alignments( bool with_alignments )
+{
+	_mut_params->set_with_alignments(with_alignments);
+}
+
+inline void ae_individual::set_with_HT( bool with_HT )
+{
+	_mut_params->set_with_HT(with_HT);
+}
+
+inline void ae_individual::set_repl_HT_with_close_points( bool repl_HT_with_close_points)
+{
+	_mut_params->set_repl_HT_with_close_points(repl_HT_with_close_points);
+}
+
+inline void ae_individual::set_HT_ins_rate( double HT_ins_rate )
+{
+	_mut_params->set_HT_ins_rate(HT_ins_rate);
+}
+
+inline void ae_individual::set_HT_repl_rate( double HT_repl_rate )
+{
+	_mut_params->set_HT_repl_rate(HT_repl_rate);
+}
+
+inline void ae_individual::set_repl_HT_detach_rate( double repl_HT_detach_rate )
+{
+	_mut_params->set_repl_HT_detach_rate(repl_HT_detach_rate);
+}
+
+
 inline void ae_individual::set_with_stochasticity( bool with_stoch )
 {
   _with_stochasticity = with_stoch;
