@@ -1,17 +1,18 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
+//          Aevol - An in silico experimental evolution platform
 //
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
+// ****************************************************************************
 // 
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,16 +20,13 @@
 // GNU General Public License for more details.
 // 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
 
-/*! \class ae_selection
-    \brief
-*/
- 
- 
+
+
 // =================================================================
 //                              Libraries
 // =================================================================
@@ -102,7 +100,7 @@ ae_selection::~ae_selection( void )
 void ae_selection::step_to_next_generation( void )
 {
   // To create the new generation, we must create nb_indivs new individuals
-  // (offsprings) and "kill" the existing ones.
+  // (offspring) and "kill" the existing ones.
   // The number of offspring on a given individual will be given by a stochastic
   // process biased on it's fitness value (the selection process).
   // There are 3 possible selection schemes :
@@ -655,261 +653,14 @@ ae_individual* ae_selection::do_replication( ae_individual* parent, int32_t inde
     new_indiv->set_grid_cell( _exp_m->get_spatial_structure()->get_grid_cell( x, y ) );
   }
   
-  
   // ===========================================================================
-  //  3) Perform transfer
-  // ===========================================================================
-  int32_t nb_indivs = _exp_m->get_pop()->get_nb_indivs();
-  
-  // -----------------------------------
-  //  a) Insertion transfer
-  // -----------------------------------
-  if ( _prng->random() < _exp_m->get_HT_ins_rate() )
-  {
-    // Insertion transfer
-    // Requirements:
-    //    * A circular exogenote => an alignment on the donor chromosome
-    //    * An alignment between the exogenote and the endogenote
-    
-    // 1) Draw a random donor (uniform drawing).
-    // We use the rank because indivs are sorted by rank (1 for the worst, POP_SIZE for the best).
-    ae_individual * donor = NULL;
-    do donor = _exp_m->get_pop()->get_indiv_by_rank( _prng->random( nb_indivs ) + 1 );
-    while ( donor == parent );
-    
-    // 2) Look for an alignment within the donor genome
-    ae_vis_a_vis* alignment_1   = NULL;
-    ae_dna*       donor_dna     = donor->get_genetic_unit( 0 )->get_dna();
-    ae_dna*       new_indiv_dna = new_indiv->get_genetic_unit( 0 )->get_dna();
-    int32_t       nb_pairs_1    = (int32_t)( ceil( donor_dna->get_length() * parent->get_neighbourhood_rate() ) );
-    
-    alignment_1 = donor_dna->search_alignment( donor_dna, nb_pairs_1, DIRECT );
-    
-    if ( alignment_1 != NULL )
-    {
-      // 3) Make a copy of the sequence to be transferred (the exogenote)
-      ae_genetic_unit* exogenote = donor_dna->copy_into_new_GU( alignment_1->get_i_1(), alignment_1->get_i_2() );
-      
-      // 4) Look for an alignments between the exogenote and the endogenote
-      ae_vis_a_vis* alignment_2 = NULL;
-      int32_t       nb_pairs_2  = (int32_t)( ceil( new_indiv_dna->get_length() * parent->get_neighbourhood_rate() ) );
-      
-      alignment_2 = exogenote->get_dna()->search_alignment( new_indiv_dna, nb_pairs_2, BOTH_SENSES );
-      
-      if ( alignment_2 != NULL )
-      {
-        //~ FILE * logfile = fopen( "transfer.out", "w" );
-        //~ fprintf( logfile, "DONOR:\n%s\n\n\n", donor_dna->get_data() );
-        //~ fprintf( logfile, "EXOGENOTE:\n%s\n\n\n", exogenote->get_dna()->get_data() );
-        //~ fprintf( logfile, "RECIPIENT:\n%s\n\n\n", new_indiv_dna->get_data() );
-        //~ #ifdef DEBUG
-          //~ ae_common::sim->get_logs()->flush();
-          //~ new_indiv->assert_promoters();
-          //~ new_indiv->assert_promoters_order();
-        //~ #endif
-        
-        //~ printf( "new indiv length: %"PRId32" parent: %"PRId32" donor: %"PRId32" exogenote: %"PRId32"\n",
-                //~ new_indiv_dna->get_length(), parent->get_genetic_unit( 0 )->get_dna()->get_length(), donor_dna->get_length(), exogenote->get_dna()->get_length() );
-        
-        //~ printf( "************************************************************\n" );
-        //~ donor->get_genetic_unit( 0 )->print_rnas();
-        //~ printf( "************************************************************\n" );
-        //~ exogenote->print_rnas();
-        //~ printf( "************************************************************\n" );
-        
-        int32_t genome_length_before  = new_indiv_dna->get_length();
-        int32_t genome_length_after   = new_indiv_dna->get_length() + exogenote->get_dna()->get_length();
-        
-        if ( genome_length_after > new_indiv->get_max_genome_length() )
-        {
-          #warning LOG
-          //~ if ( _exp_m->is_logged( LOG_BARRIER ) == true )
-          //~ {
-            //~ // Write an entry in the barrier log file
-            //~ fprintf(  ae_common::rec_params->get_log( LOG_BARRIER ), "%"PRId32" %"PRId32" INS_TRANSFER %"PRId32" %"PRId32" %"PRId32"\n",
-                      //~ _exp_m->get_num_gener(),
-                      //~ new_indiv->get_index_in_population(),
-                      //~ exogenote->get_dna()->get_length(),
-                      //~ 0,
-                      //~ genome_length_before );
-          //~ }
-        }
-        else
-        {
-          new_indiv_dna->insert_GU( exogenote, alignment_2->get_i_2(), alignment_2->get_i_1(), (alignment_2->get_sense() == INDIRECT) );
-          //~ fprintf( logfile, "RESULT:\n%s\n\n\n", new_indiv_dna->get_data() );
-          //~ fflush( logfile );
-          
-
-          // Write a line in transfer logfile
-          #warning LOG
-          //~ if ( ae_common::rec_params->is_logged( LOG_TRANSFER ) == true )
-          //~ {
-            //~ fprintf(  ae_common::rec_params->get_log( LOG_TRANSFER ),
-                      //~ "%"PRId32" %"PRId32" %"PRId32" %"PRId8" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId16" %"PRId32" %"PRId32" %"PRId16"\n",
-                      //~ _exp_m->get_num_gener(),
-                      //~ new_indiv->get_index_in_population(),
-                      //~ donor->get_index_in_population(),
-                      //~ 0, // Transfer type
-                      //~ exogenote->get_dna()->get_length(),
-                      //~ 0,
-                      //~ genome_length_before,
-                      //~ new_indiv_dna->get_length(),
-                      //~ alignment_1->get_i_1(),
-                      //~ alignment_1->get_i_2(),
-                      //~ alignment_1->get_score(),
-                      //~ alignment_2->get_i_1(),
-                      //~ alignment_2->get_i_2(),
-                      //~ alignment_2->get_score() );
-          //~ }
-    
-          #ifdef BIG_DEBUG
-            ae_common::sim->get_logs()->flush();
-            new_indiv->assert_promoters();
-            new_indiv->assert_promoters_order();
-          #endif
-        }
-        
-        delete alignment_2;
-      }
-      
-      delete exogenote;
-      delete alignment_1;
-    }
-  }
-  
-  // -----------------------------------
-  //  b) Replacement transfer
-  // -----------------------------------
-  if ( _prng->random() < _exp_m->get_HT_repl_rate() )
-  {
-    // Replacement transfer
-    // Requirements:
-    //    * 2 distinct alignments between the (linear) exogenote and the endogenote
-    
-    // 1) Draw a random donor (uniform drawing).
-    // We use the rank because indivs are sorted by rank (1 for the worst, POP_SIZE for the best).
-    ae_individual * donor = NULL;
-    do donor = _exp_m->get_pop()->get_indiv_by_rank( _prng->random( nb_indivs ) + 1 );
-    while ( donor == parent );
-    
-    // 2) Look for an alignment between the parent genome and the donor genome
-    ae_vis_a_vis* alignment_1   = NULL;
-    ae_vis_a_vis* alignment_2   = NULL;
-    ae_dna*       parent_dna    = parent->get_genetic_unit( 0 )->get_dna();
-    ae_dna*       donor_dna     = donor->get_genetic_unit( 0 )->get_dna();
-    ae_dna*       new_indiv_dna = new_indiv->get_genetic_unit( 0 )->get_dna();
-    ae_sense      sense         = (_prng->random() < 0.5) ? DIRECT : INDIRECT;
-    int32_t       nb_pairs_1    = (int32_t)( ceil( new_indiv_dna->get_length() * parent->get_neighbourhood_rate() ) );
-    int32_t       nb_pairs_2    = (int32_t)( ceil( new_indiv_dna->get_length() * parent->get_neighbourhood_rate() ) );
-    
-    alignment_1 = parent_dna->search_alignment( donor_dna, nb_pairs_1, sense );
-    
-    if ( alignment_1 != NULL )
-    {
-      // Look for a second alignement between the parent and the donor (must be different from alignment_1)
-      while ( alignment_2 == NULL && nb_pairs_2 > 0 )
-      {
-        alignment_2 = parent_dna->search_alignment( donor_dna, nb_pairs_2, sense );
-        
-        // Forbid the replacement of the whole genome of the parent
-        if ( alignment_2 != NULL && alignment_2->get_i_1() == alignment_1->get_i_1() )
-        {
-          delete alignment_2;
-          alignment_2 = NULL;
-        }
-      }
-    
-    
-      // If both alignments were found, proceed to the transfer
-      if ( alignment_2 != NULL )
-      {
-        int32_t genome_length_before  = new_indiv_dna->get_length();
-        int32_t genome_length_after   = new_indiv_dna->get_length();
-        int32_t exogenote_length      = ae_utils::mod( alignment_2->get_i_2() - alignment_1->get_i_2() - 1, donor_dna->get_length() ) + 1;
-        int32_t replaced_seq_length   = ae_utils::mod( alignment_2->get_i_1() - alignment_1->get_i_1() - 1, genome_length_before ) + 1;
-        
-        if ( genome_length_after < new_indiv->get_min_genome_length() || genome_length_after > new_indiv->get_max_genome_length() )
-        {
-          #warning LOG
-          //~ if ( ae_common::rec_params->is_logged( LOG_BARRIER ) == true )
-          //~ {
-            //~ // Write an entry in the barrier log file
-            //~ fprintf(  ae_common::rec_params->get_log( LOG_BARRIER ), "%"PRId32" %"PRId32" REPL_TRANSFER %"PRId32" %"PRId32" %"PRId32"\n",
-                      //~ _exp_m->get_num_gener(),
-                      //~ new_indiv->get_index_in_population(),
-                      //~ exogenote_length,
-                      //~ replaced_seq_length,
-                      //~ genome_length_before );
-          //~ }
-        }
-        
-        // 3) Make a copy of the sequence to be transferred (the exogenote)
-        ae_genetic_unit* exogenote = NULL;
-        if ( sense == DIRECT )
-        {
-          exogenote = donor_dna->copy_into_new_GU( alignment_1->get_i_2(), alignment_2->get_i_2() );
-        }
-        else
-        {
-          exogenote = donor_dna->copy_into_new_GU( alignment_2->get_i_2(), alignment_1->get_i_2() );
-        }
-        
-        // Delete the sequence to be replaced
-        new_indiv_dna->do_deletion( alignment_1->get_i_1(), alignment_2->get_i_1() );
-        
-        //~ printf( "intermediate genome length : %"PRId32"\n", new_indiv_dna->get_length() );
-        
-        // Insert the transfered sequence
-        if ( alignment_1->get_i_1() < alignment_2->get_i_1() )
-        {
-          new_indiv_dna->insert_GU( exogenote, alignment_1->get_i_1(), 0, sense == INDIRECT );
-        }
-        else
-        {
-          new_indiv_dna->insert_GU( exogenote, 0, 0, sense == INDIRECT );
-        }
-        
-
-        // Write a line in transfer logfile
-        #warning LOG
-        //~ if ( ae_common::rec_params->is_logged( LOG_TRANSFER ) == true )
-        //~ {
-          //~ fprintf(  ae_common::rec_params->get_log( LOG_TRANSFER ),
-                    //~ "%"PRId32" %"PRId32" %"PRId32" %"PRId8" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId32" %"PRId16" %"PRId32" %"PRId32" %"PRId16"\n",
-                    //~ _exp_m->get_num_gener(),
-                    //~ new_indiv->get_index_in_population(),
-                    //~ donor->get_index_in_population(),
-                    //~ 1, // Transfer type
-                    //~ exogenote->get_dna()->get_length(),
-                    //~ replaced_seq_length,
-                    //~ genome_length_before,
-                    //~ new_indiv_dna->get_length(),
-                    //~ alignment_1->get_i_1(),
-                    //~ alignment_1->get_i_2(),
-                    //~ alignment_1->get_score(),
-                    //~ alignment_2->get_i_1(),
-                    //~ alignment_2->get_i_2(),
-                    //~ alignment_2->get_score() );
-        //~ }
-        
-        delete exogenote;
-        delete alignment_2;
-      }
-      
-      delete alignment_1;
-    }
-  }
-  
-  
-  // ===========================================================================
-  //  4) Perform rearrangements and mutations
+  //  3) Perform transfer, rearrangements and mutations
   // ===========================================================================
   if ( ! new_indiv->get_allow_plasmids() )
   {
     ae_genetic_unit* chromosome = new_indiv->get_genetic_unit_list()->get_first()->get_obj();
     
-    chromosome->get_dna()->perform_mutations();
+    chromosome->get_dna()->perform_mutations( parent->get_id());
     
     if ( new_indiv->get_replic_report() != NULL )
     {
@@ -934,7 +685,7 @@ ae_individual* ae_selection::do_replication( ae_individual* parent, int32_t inde
       {
         gen_unit = gen_unit_node->get_obj();
         
-        gen_unit->get_dna()->perform_mutations();
+        gen_unit->get_dna()->perform_mutations( parent->get_id() );
         
         if ( new_indiv->get_replic_report() != NULL )
         {
@@ -953,7 +704,7 @@ ae_individual* ae_selection::do_replication( ae_individual* parent, int32_t inde
       {
         gen_unit = gen_unit_node->get_obj();
         
-        gen_unit->get_dna()->perform_mutations();
+        gen_unit->get_dna()->perform_mutations( parent->get_id() );
         
         if ( new_indiv->get_replic_report() != NULL )
         {
@@ -968,20 +719,20 @@ ae_individual* ae_selection::do_replication( ae_individual* parent, int32_t inde
 
   
   // ===========================================================================
-  //  5) Evaluate new individual
+  //  4) Evaluate new individual
   // ===========================================================================
   new_indiv->evaluate();
   
   
   // ===========================================================================
-  //  6) Compute statistics
+  //  5) Compute statistics
   // ===========================================================================
   new_indiv->compute_statistical_data();
   
   
   #ifdef BIG_DEBUG
-  ae_common::sim->get_logs()->flush();
-  new_indiv->assert_promoters();
+  // ae_common::sim->get_logs()->flush();   // ae_common::sim is obsolete in version 4
+  //  new_indiv->assert_promoters();
   #endif
   
   
