@@ -1,32 +1,30 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
-//
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+//          Aevol - An in silico experimental evolution platform
 //
+// ****************************************************************************
+// 
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
 
-/** \class
- *  \brief
- */
 
 
 // =================================================================
@@ -163,7 +161,7 @@ ae_stats::ae_stats( ae_exp_manager* exp_m,
         }
       }
     }
-    delete old_file_name;
+    delete [] old_file_name;
   }
   else // ancstat case
   {
@@ -432,7 +430,7 @@ void ae_stats::write_headers( bool ancstats_stats /* = false */ )
   // ---------------------------------------
   for ( int8_t chrom_or_GU = 0 ; chrom_or_GU < NB_CHROM_OR_GU ; chrom_or_GU++ )
   {
-    if( ancstats_stats)
+    if( ancstats_stats )
     {
       write_header( _stat_files[chrom_or_GU][BEST][BP_STATS], "-------------------------------------------" );
       write_header( _stat_files[chrom_or_GU][BEST][BP_STATS], " Lineage individuals non-coding statistics " );
@@ -480,8 +478,13 @@ void ae_stats::write_headers( bool ancstats_stats /* = false */ )
         write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "Number of bp not included in any RNA", key++ );
         write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "Number of bp not included in any coding RNA", key++ );
         write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "Number of bp not included in any non coding RNA", key++ );
-        
-        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], ""); 
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "Number of non essential bp", key++ );
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "Number of non essential bp including non fonctional genes", key++ );
+
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "");
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "NOTE: a bp is considered \"essential\" when it is part of any [functional] CDS");
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "  or any promoter or terminator corresponding to an RNA transcribing a [functional] CDS.");
+        write_header( _stat_files[chrom_or_GU][best_or_glob][BP_STATS], "");
       }
     }
   }
@@ -653,7 +656,7 @@ void ae_stats::set_file_names( const char * prefix, bool one_lambda_indiv_only )
   status = mkdir( STATS_DIR, 0755 );
   if ( (status == -1) && (errno != EEXIST) )
   {
-    err( EXIT_FAILURE, STATS_DIR, errno );
+    err( EXIT_FAILURE, STATS_DIR );
   }
   
   const char* chrom_or_gu_name[NB_CHROM_OR_GU]    = { "", "_chromosome", "_plasmids" };
