@@ -1,32 +1,30 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
-//
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+//          Aevol - An in silico experimental evolution platform
 //
+// ****************************************************************************
+// 
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
 
-/** \class
- *  \brief
- */
 
 
 // =================================================================
@@ -51,6 +49,7 @@
 #include <ae_gaussian.h>
 #include <ae_env_segment.h>
 #include <ae_point_2d.h>
+#include <ae_align.h>
 
 //~ #ifdef __X11
   //~ #include <ae_individual_X11.h>
@@ -568,6 +567,23 @@ void param_loader::interpret_line( f_line* line, int32_t _cur_line )
       exit( EXIT_FAILURE ); 
     }
   }
+  else if ( strcmp( line->words[0], "REPL_TRANSFER_WITH_CLOSE_POINTS" ) == 0 )
+  {
+    if ( strncmp( line->words[1], "true", 4 ) == 0 )
+    {
+      _param_values->set_repl_HT_with_close_points( true );
+    }
+    else if ( strncmp( line->words[1], "false", 5 ) == 0 )
+    {
+      _param_values->set_repl_HT_with_close_points( false );
+    }
+    else
+    {
+      printf( "ERROR in param file \"%s\" on line %"PRId32" : unknown transfer option (use true/false).\n",
+              _param_file_name, _cur_line );
+      exit( EXIT_FAILURE ); 
+    }
+  }
   else if ( strcmp( line->words[0], "SWAP_GUS" ) == 0 )
   {
     if ( strncmp( line->words[1], "true", 4 ) == 0 )
@@ -593,6 +609,10 @@ void param_loader::interpret_line( f_line* line, int32_t _cur_line )
   {
     _param_values->set_HT_repl_rate( atof( line->words[1] ) );
   }
+  else if ( strcmp( line->words[0], "REPL_TRANSFER_DETACH_RATE" ) == 0 )
+  {
+    _param_values->set_repl_HT_detach_rate( atof( line->words[1] ) );
+  }
   else if ( strcmp( line->words[0], "TRANSLATION_COST" ) == 0 )
   {
     _param_values->set_translation_cost( atof( line->words[1] ) );
@@ -914,19 +934,19 @@ void param_loader::read_file( void )
 
 void param_loader::load( ae_exp_manager* exp_m, bool verbose )
 {
-	// Initialize _prng
+  // Initialize _prng
   _prng = new ae_jumping_mt( _param_values->_seed );
   
   // Initialize _mut_prng and _stoch_prng :
   // if mut_seed (respectively stoch_seed) not given in param.in, choose it at random
   if ( _param_values->_mut_seed == 0 ) {
-		_param_values->set_mut_seed( _prng->random( 1000000 ) );
-	}
-	if ( _param_values->_stoch_seed == 0 ) {
-		_param_values->set_stoch_seed( _prng->random( 1000000 ) );
-	}
-	_mut_prng = new ae_jumping_mt( _param_values->_mut_seed );
-	_stoch_prng = new ae_jumping_mt( _param_values->_stoch_seed );
+    _param_values->set_mut_seed( _prng->random( 1000000 ) );
+  }
+  if ( _param_values->_stoch_seed == 0 ) {
+    _param_values->set_stoch_seed( _prng->random( 1000000 ) );
+  }
+  _mut_prng = new ae_jumping_mt( _param_values->_mut_seed );
+  _stoch_prng = new ae_jumping_mt( _param_values->_stoch_seed );
   
   // Create aliases (syntaxic sugars)
   ae_exp_setup*       exp_s     = exp_m->get_exp_s();
@@ -960,8 +980,10 @@ void param_loader::load( ae_exp_manager* exp_m, bool verbose )
   
   // ----------------------------------------------------------------- Transfer
   exp_s->set_with_HT( _param_values->_with_HT );
+  exp_s->set_repl_HT_with_close_points( _param_values->get_repl_HT_with_close_points() );
   exp_s->set_HT_ins_rate( _param_values->_HT_ins_rate );
   exp_s->set_HT_repl_rate( _param_values->_HT_repl_rate );
+  exp_s->set_repl_HT_detach_rate( _param_values->_repl_HT_detach_rate );
   
   // ----------------------------------------------------------------- Plasmids
   exp_s->set_with_plasmids( _param_values->_allow_plasmids );
@@ -1004,11 +1026,17 @@ void param_loader::load( ae_exp_manager* exp_m, bool verbose )
   env->set_sampling( _param_values->get_env_sampling() );
   
   // Set the environment segmentation
-  env->set_segmentation( _param_values->get_env_axis_nb_segments(),
+  if( (_param_values->get_env_axis_features() != NULL) && (_param_values->get_env_axis_segment_boundaries() != NULL)  )
+    {
+      // if param.in contained a line starting with ENV_AXIS_FEATURES,
+      // we use the values indicated on this line
+      env->set_segmentation( _param_values->get_env_axis_nb_segments(),
                          _param_values->get_env_axis_segment_boundaries(),
                          _param_values->get_env_axis_features(),
                          _param_values->get_env_axis_separate_segments() );
-  
+    } // else we leave the ae_environment as it is by default (one "metabolic" segment from X_MIN to XMAX)
+
+
   // Set environmental variation
   if ( _param_values->get_env_var_method() != NO_VAR )
   {
@@ -1050,8 +1078,10 @@ void param_loader::load( ae_exp_manager* exp_m, bool verbose )
   param_mut->set_with_4pts_trans( _param_values->get_with_4pts_trans() );
   param_mut->set_with_alignments( _param_values->get_with_alignments() );
   param_mut->set_with_HT( _param_values->get_with_HT() );
+  param_mut->set_repl_HT_with_close_points( _param_values->get_repl_HT_with_close_points() );
   param_mut->set_HT_ins_rate( _param_values->get_HT_ins_rate() );
   param_mut->set_HT_repl_rate( _param_values->get_HT_repl_rate() );
+  param_mut->set_repl_HT_detach_rate( _param_values->get_repl_HT_detach_rate() );
   param_mut->set_duplication_rate( _param_values->get_duplication_rate() );
   param_mut->set_deletion_rate( _param_values->get_deletion_rate() );
   param_mut->set_translocation_rate( _param_values->get_translocation_rate() );
@@ -1094,6 +1124,7 @@ void param_loader::load( ae_exp_manager* exp_m, bool verbose )
         // Add it to the list
         pop->add_indiv( clone );
       }
+      pop->sort_individuals();
     }
     else // if ( ! CLONE )
     {
@@ -1134,6 +1165,7 @@ void param_loader::load( ae_exp_manager* exp_m, bool verbose )
         // Add it to the list
         pop->add_indiv( clone );
       }
+      pop->sort_individuals();
     }
     else // if ( ! CLONE )
     {
