@@ -1,26 +1,27 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
-//
-// Copyright (C) 2004  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+//          Aevol - An in silico experimental evolution platform
 //
+// ****************************************************************************
+// 
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
 
@@ -135,8 +136,10 @@ class params : public ae_object
     inline bool   get_with_4pts_trans( void ) const;
     inline bool   get_with_alignments( void ) const;
     inline bool   get_with_HT( void ) const;
+    inline bool   get_repl_HT_with_close_points( void ) const;
     inline double get_HT_ins_rate( void ) const;
     inline double get_HT_repl_rate( void ) const;
+    inline double get_repl_HT_detach_rate( void ) const;
 
     // ------------------------------ Rearrangement rates (without alignements)
     inline double get_duplication_rate( void ) const;
@@ -294,8 +297,10 @@ class params : public ae_object
     inline void set_with_4pts_trans( bool with_4pts_trans );
     inline void set_with_alignments( bool with_alignments );
     inline void set_with_HT( bool with_HT );
+    inline void set_repl_HT_with_close_points( bool repl_HT_with_close_points);
     inline void set_HT_ins_rate( double HT_ins_rate );
     inline void set_HT_repl_rate( double HT_repl_rate );
+    inline void set_repl_HT_detach_rate( double repl_HT_detach_rate );
 
     // ------------------------------ Rearrangement rates (without alignements)
     inline void set_duplication_rate( double duplication_rate );
@@ -493,8 +498,10 @@ class params : public ae_object
     bool    _with_4pts_trans;
     bool    _with_alignments;
     bool    _with_HT;
+    bool    _repl_HT_with_close_points;
     double  _HT_ins_rate;
     double  _HT_repl_rate;
+    double  _repl_HT_detach_rate;
 
     // ------------------------------ Rearrangement rates (without alignements)
     double _duplication_rate;
@@ -780,6 +787,11 @@ inline bool params::get_with_HT( void ) const
   return _with_HT;
 }
 
+inline bool params::get_repl_HT_with_close_points( void ) const
+{
+  return _repl_HT_with_close_points;
+}
+
 inline double params::get_HT_ins_rate( void ) const
 {
   return _HT_ins_rate;
@@ -790,6 +802,11 @@ inline double params::get_HT_repl_rate( void ) const
   return _HT_repl_rate;
 }
 
+inline double params::get_repl_HT_detach_rate( void ) const
+{
+  return _repl_HT_detach_rate;
+}
+
 // ---------------------------------- Rearrangement rates (without alignements)
 inline double params::get_duplication_rate( void ) const
 {
@@ -1260,6 +1277,11 @@ inline void params::set_with_HT( bool with_HT )
   _with_HT = with_HT;
 }
 
+inline void params::set_repl_HT_with_close_points( bool repl_HT_with_close_points)
+{
+  _repl_HT_with_close_points = repl_HT_with_close_points;
+}
+
 inline void params::set_HT_ins_rate( double HT_ins_rate )
 {
   _HT_ins_rate = HT_ins_rate;
@@ -1270,6 +1292,11 @@ inline void params::set_HT_repl_rate( double HT_repl_rate )
   _HT_repl_rate = HT_repl_rate;
 }
 
+inline void params::set_repl_HT_detach_rate( double repl_HT_detach_rate )
+{
+  _repl_HT_detach_rate = repl_HT_detach_rate;
+}
+
 // ---------------------------------- Rearrangement rates (without alignements)
 inline void params::set_duplication_rate( double duplication_rate )
 {
