@@ -1,17 +1,18 @@
-//*****************************************************************************
+// ****************************************************************************
 //
-//                         aevol - Artificial Evolution
+//          Aevol - An in silico experimental evolution platform
 //
-// Copyright (C) 2009  LIRIS.
-// Web: https://liris.cnrs.fr/
-// E-mail: carole.knibbe@liris.cnrs.fr
-// Original Authors : Guillaume Beslon, Carole Knibbe, Virginie Lefort
-//                    David Parsons
+// ****************************************************************************
 // 
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
+// Copyright: See the AUTHORS file provided with the package or <www.aevol.fr>
+// Web: http://www.aevol.fr/
+// E-mail: See <http://www.aevol.fr/contact/>
+// Original Authors : Guillaume Beslon, Carole Knibbe, David Parsons
+// 
+// This program is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,18 +20,13 @@
 // GNU General Public License for more details.
 // 
 // You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// 
 //*****************************************************************************
 
 
-/** \class
- *  \brief
- */
- 
- 
- 
- 
+
+
 // =================================================================
 //                              Libraries
 // =================================================================
@@ -232,7 +228,7 @@ int main(int argc, char** argv)
   char pop_file_name[255];
   sprintf( pop_file_name, POP_FNAME_FORMAT, end_gener );
 
-  tree = new ae_tree( exp_manager, pop_file_name, tree_file_name );
+  tree = new ae_tree( exp_manager, tree_file_name );
 
   if ( verbose )
   {
@@ -334,13 +330,12 @@ int main(int argc, char** argv)
       
       sprintf( pop_file_name,       POP_FNAME_FORMAT,       num_gener );
       
-      tree = new ae_tree( exp_manager, pop_file_name, tree_file_name );
+      tree = new ae_tree( exp_manager, tree_file_name );
     }
     
-
     // Copy the replication report of the ancestor
     reports[i] = new ae_replication_report( *(tree->get_report_by_index(num_gener, indices[i + 1])) );
-
+    
     // Retreive the index and rank of the next ancestor from the report
     indices[i] = reports[i]->get_parent_id();
   }
@@ -451,7 +446,7 @@ int main(int argc, char** argv)
   	  #else
     	exp_manager_backup = new ae_exp_manager();
       #endif
-      exp_manager_backup->load( num_gener, false, true );
+      exp_manager_backup->load( num_gener, false, true, false );
   
       // Copy the ancestor from the backup
       // NB : The list of individuals is sorted according to the index
@@ -474,6 +469,14 @@ int main(int argc, char** argv)
       rep = (ae_dna_replic_report *) report_node->get_obj();
       unit = (ae_genetic_unit *) gen_unit_node->get_obj();
       
+      mut_node = rep->get_HT()->get_first(); 
+      while ( mut_node != NULL )
+      {
+        mut = (ae_mutation *) mut_node->get_obj();
+        (unit->get_dna())->undergo_this_mutation( mut );
+        mut_node = mut_node->get_next();
+      }
+      
       mut_node = rep->get_rearrangements()->get_first(); 
       while ( mut_node != NULL )
       {
